unit SunAnimation2U;

//Animation of a sun image using an image list
//Form has a background made up of tiled Union Jacks

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QTypes, QExtCtrls, QImgList;

type
  TfrmMain = class(TForm)
    Timer1: TTimer;
    Imgs: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    BkGnd, IntactBkGnd: TBitmap; //The background and background backup bitmaps
    SunPos: TPoint; //defaults to (0, 0)
    procedure SetupBackground;
    function SunRect(TopLeft: TPoint): TRect;
  public
    function MoveSun: TRect;
    function DrawSun: TRect;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.xfm}

uses
  UnionJack2;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  BkGnd := TBitmap.Create;
  IntactBkGnd := TBitmap.Create;
  SetupBackground; //draw offscreen background (& background backup)
  DrawSun; //start with sun at top left of form
  Timer1.Enabled := True;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  IntactBkGnd.Free;
  BkGnd.Free
end;

procedure TfrmMain.FormPaint(Sender: TObject);
begin
  //Copy as much of the bitmap as will fit on the form
  Canvas.CopyRect(ClientRect, BkGnd.Canvas, ClientRect)
end;

procedure TfrmMain.SetupBackground;
var
  TmpBmp: TBitmap;
  X, Y: Integer;
const
  XGap = 2;
  YGap = 2;
begin
  TmpBmp := TBitmap.Create;
  try
    //Draw a Union Jack
    TmpBmp.Width := 100;
    TmpBmp.Height := 50;
    DrawUnionJack(TmpBmp.Canvas, Rect(0, 0, TmpBmp.Width, TmpBmp.Height));
    //Tile Union Jack on a screen-sized bitmap
    BkGnd.Canvas.Brush.Color := Color;
    BkGnd.Width := Screen.Width;
    BkGnd.Height := Screen.Height;
    Y := 0;
    while Y < BkGnd.Height do
    begin
      X := 0;
      while X < BkGnd.Width do
      begin
        BkGnd.Canvas.Draw(X, Y, TmpBmp);
        Inc(X, TmpBmp.Width + XGap);
      end;
      Inc(Y, TmpBmp.Height + YGap);
    end;
  finally
    TmpBmp.Free
  end;
  //Duplicate tiled background in backup bitmap
  IntactBkGnd.Assign(BkGnd);
end;

procedure TfrmMain.Timer1Timer(Sender: TObject);
var
  EraseRect, DrawRect, UpdateRect: TRect;
begin
  EraseRect := MoveSun;
  DrawRect := DrawSun;
  UnionRect(UpdateRect, EraseRect, DrawRect);
  Canvas.CopyRect(UpdateRect, BkGnd.Canvas, UpdateRect);
end;

function TfrmMain.DrawSun: TRect;
begin
  //Draw sun in new position
  Imgs.Draw(BkGnd.Canvas, SunPos.X, SunPos.Y, 0);
  Result := SunRect(SunPos);
end;

var
  XInc: Integer = 5;
  YInc: Integer = 5;

function TfrmMain.MoveSun: TRect;
begin
  Result := SunRect(SunPos);
  if ((XInc > 0) and (Result.Right > ClientWidth)) or
     ((XInc < 0) and (SunPos.X < 0)) then
    XInc := -XInc;
  Inc(SunPos.X, XInc);
  if SunPos.X > ClientWidth then
    SunPos.X := ClientWidth;
  if ((YInc > 0) and (Result.Bottom > ClientHeight)) or
     ((YInc < 0) and (SunPos.Y < 0)) then
    YInc := -YInc;
  Inc(SunPos.Y, YInc);
  if SunPos.Y > ClientHeight then
    SunPos.Y := ClientHeight;
  //Overdraw last sun image with original background
  BkGnd.Canvas.CopyMode := cmSrcCopy;
  BkGnd.Canvas.CopyRect(Result, IntactBkGnd.Canvas, Result);
end;

function TfrmMain.SunRect(TopLeft: TPoint): TRect;
begin
  Result := Rect(TopLeft.X, TopLeft.Y,
    TopLeft.X + Imgs.Width, TopLeft.Y + Imgs.Height);
end;

end.
