unit SunAnimationU;

//Animation of a sun image using an image list
//Form has a background made up of tiled Union Jacks

interface

uses
{$ifdef MSWINDOWS}
  Windows,
{$endif}
{$ifdef LINUX}
  Xlib,
{$endif}
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QTypes, QExtCtrls, QImgList;

type
  TfrmMain = class(TForm)
    Timer1: TTimer;
    Imgs: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    BkGnd, IntactBkGnd: TBitmap; //The background and background backup bitmaps
    SunPos: TPoint; //defaults to (0, 0)
    procedure SetupBackground;
    function SunRect(TopLeft: TPoint): TRect;
  public
    function MoveSun(X, Y: Integer): TRect;
    function DrawSun: TRect;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.xfm}

uses
{$ifdef LINUX}
  XKeyHelper,
{$endif}
  UnionJack2;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  BkGnd := TBitmap.Create;
  IntactBkGnd := TBitmap.Create;
  SetupBackground; //draw offscreen background (& background backup)
  DrawSun; //start with sun at top left of form
  Timer1.Enabled := True;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  IntactBkGnd.Free;
  BkGnd.Free
end;

procedure TfrmMain.FormPaint(Sender: TObject);
begin
  //Copy as much of the bitmap as will fit on the form
  Canvas.CopyRect(ClientRect, BkGnd.Canvas, ClientRect)
end;

procedure TfrmMain.SetupBackground;
var
  TmpBmp: TBitmap;
  X, Y: Integer;
const
  XGap = 2;
  YGap = 2;
begin
  TmpBmp := TBitmap.Create;
  try
    //Draw a Union Jack
    TmpBmp.Width := 100;
    TmpBmp.Height := 50;
    DrawUnionJack(TmpBmp.Canvas, Rect(0, 0, TmpBmp.Width, TmpBmp.Height));
    //Tile Union Jack on a screen-sized bitmap
    BkGnd.Canvas.Brush.Color := Color;
    BkGnd.Width := Screen.Width;
    BkGnd.Height := Screen.Height;
    Y := 0;
    while Y < BkGnd.Height do
    begin
      X := 0;
      while X < BkGnd.Width do
      begin
        BkGnd.Canvas.Draw(X, Y, TmpBmp);
        Inc(X, TmpBmp.Width + XGap);
      end;
      Inc(Y, TmpBmp.Height + YGap);
    end;
  finally
    TmpBmp.Free
  end;
  //Duplicate tiled background in backup bitmap
  IntactBkGnd.Assign(BkGnd);
end;

procedure TfrmMain.Timer1Timer(Sender: TObject);
var
  X, Y: Integer;
  EraseRect, DrawRect, UpdateRect: TRect;
const
  XInc = 5;
  YInc = 5;
begin
  X := 0;
  Y := 0;
{$ifdef MSWINDOWS}
  if GetKeyState(VK_LEFT) and $80 > 0 then
{$endif}
{$ifdef LINUX}
  if IsKeyDown(XK_Left) then
{$endif}
    Dec(X, XInc);
{$ifdef MSWINDOWS}
  if GetKeyState(VK_RIGHT) and $80 > 0 then
{$endif}
{$ifdef LINUX}
  if IsKeyDown(XK_Right) then
{$endif}
    Inc(X, XInc);
{$ifdef MSWINDOWS}
  if GetKeyState(VK_UP) and $80 > 0 then
{$endif}
{$ifdef LINUX}
  if IsKeyDown(XK_Up) then
{$endif}
    Dec(Y, YInc);
{$ifdef MSWINDOWS}
  if GetKeyState(VK_DOWN) and $80 > 0 then
{$endif}
{$ifdef LINUX}
  if IsKeyDown(XK_Down) then
{$endif}
    Inc(Y, YInc);
  if (X <> 0) or (Y <> 0) then
  begin
    EraseRect := MoveSun(X, Y);
    DrawRect := DrawSun;
    UnionRect(UpdateRect, EraseRect, DrawRect);
    Canvas.CopyRect(UpdateRect, BkGnd.Canvas, UpdateRect);
  end;
end;

function TfrmMain.DrawSun: TRect;
begin
  //Draw sun in new position
  Imgs.Draw(BkGnd.Canvas, SunPos.X, SunPos.Y, 0);
  Result := SunRect(SunPos);
end;

function TfrmMain.MoveSun(X, Y: Integer): TRect;
begin
  Result := SunRect(SunPos);
  Inc(SunPos.X, X);
  Inc(SunPos.Y, Y);
  //Overdraw last sun image with original background
  BkGnd.Canvas.CopyMode := cmSrcCopy;
  BkGnd.Canvas.CopyRect(Result, IntactBkGnd.Canvas, Result);
end;

function TfrmMain.SunRect(TopLeft: TPoint): TRect;
begin
  Result := Rect(TopLeft.X, TopLeft.Y,
    TopLeft.X + Imgs.Width, TopLeft.Y + Imgs.Height);
end;

end.
